library(shiny)
options(shiny.maxRequestSize = 10000000000000*1024^2)

ui <- pageWithSidebar( 
  headerPanel("Linear Spectral Unmixing Model v1.1"),
  sidebarPanel(
    #Panel 1
    conditionalPanel(condition="input.conditionedPanels==1", 
                     helpText("Content Panel 1"),
                     fileInput("file1", "Upload the mixed spectra file (CSV format)."),
                     tags$hr(),
                     fileInput("file2", "Upload the reference file (CSV format)."),
                     tags$hr(),
                     strong(helpText("What is the range of wavelength you would like to analyze?")),
                     textInput("lower", "Enter the lower limit:", 0),
                     textInput("upper", "Enter the upper limit:", 1500),
                     
                     tags$hr(),
                     textInput("name1", "What would you like to call Component 1?"),
                     textInput("name2", "What would you like to call Component 2?"),
                     tags$hr(),
                     actionButton("do", "Run"),
                     tags$hr(),
                     textInput("name3", "What would you like to call the output file?", "Output"),
                     tags$hr(),
                     downloadButton("downloadData","Download dataset.")	
    ),
    #Panel 2
    conditionalPanel(condition="input.conditionedPanels==2",
                     helpText("Content Panel 2"),
                     numericInput("time_point", "Enter a time point:", min = 0, value = 0) #Use Numeric Input
    ) 
  ),
  
  mainPanel(
    tabsetPanel(
      tabPanel(
        "Panel 1",
        plotOutput("color1_plot"),
        plotOutput("color2_plot"),
        plotOutput("ratio_plot"),
        value=1
      ), 
      tabPanel(
        "Panel 2", 
        plotOutput("Regression_Line"),
        verbatimTextOutput("info"),
        value=2
      ), 
      id = "conditionedPanels"
    )
  )
)


server <- function(input, output){
  
#Panel 1
  lower_lim <- reactive({
    as.numeric(input$lower)
  })
  upper_lim <- reactive({
    as.numeric(input$upper)
  })
  name1 <- reactive({ #Component 1 Name
    input$name1
  })
  name2 <- reactive({	#Component 2 Name
    input$name2
  })
  name3 <- reactive({	#Output File Name
    input$name3
  })
  mydata <- eventReactive(input$do, {		
    withProgress(message = paste("Reading ", input$file1$name),{		
      
      mixed_data <- as.data.frame(t(read.csv(input$file1$datapath, head = FALSE, sep = ",")))	#Reads in mixed data file
      
      upper_lim2 <- tail(which(mixed_data[[1]] < upper_lim()),1) #Determines the wavelength value closest to the upper limit
      lower_lim2 <- head(which(mixed_data[[1]] > lower_lim()),1) #Determines the wavelength value closest to the lower limit
      mixed_data <- mixed_data[lower_lim2:upper_lim2, ] #Removes data outside of the bounds
      wavelengths <- mixed_data[[1]] #Creates a new array of wavelengths
      mixed_data <- mixed_data[,-1] #Removes first column (wavelengths) from the mixed data
    })	
    Sys.sleep(.5)
    withProgress(message = paste("Reading ", input$file2$name),{		
      
      color <- read.csv(input$file2$datapath, head = TRUE, sep = ",") #Reads in the reference data
      upper_lim2 <- tail(which(color[[1]] < upper_lim()),1) 
      lower_lim2 <- head(which(color[[1]] > lower_lim()),1)
      color <- color[lower_lim2:upper_lim2,] #Removes the data outside of the bounds
      
      wavelengths2 <- color[[1]] #Creates a new array of wavelengths
      color1 <- color[[2]] #Creates a new array for the first reference
      color2 <- color[[3]] #Creates a new array for the second reference
    })		
    Sys.sleep(.5)
    
    r_sq_list <- NULL
    adj_r_sq_list <- NULL
    intercept_list <- NULL
    comp1_list <- NULL
    comp2_list <- NULL
    ratio_list <- NULL
    
    withProgress(message = 'Generating Data', detail = "Part 0", value = 0, {
      for(i in 1:(ncol(mixed_data))){
        
        m <- mixed_data[[i]] #Grabs data at a given time point
        
        fit <- lm(m ~ color1+ color2) #Models curve with a constant
        
        
        r_sq <- summary(fit)$r.squared #Grabs the R^2 value
        adj_r_sq <- summary(fit)$adj.r.squared #Grabs the adjusted R^2 value
        
        coeff_header <- summary(fit)$coefficients #Grabs the coefficient header
        coeff <- matrix(coeff_header[,1],dimnames = NULL) #Grabs the intercept and the component coefficients
        intercept <- coeff[1] #Intercept
        comp1 <- coeff[2] #Coefficient of component 1
        comp2 <- coeff[3] #Coefficient of compenent 2
        ratio <- comp1/comp2 #Ratio of component 1 to component 2. swap components if desired
        
        #Adds above values to lists
        r_sq_list <- rbind(r_sq_list, r_sq)
        adj_r_sq_list <- rbind(adj_r_sq_list, adj_r_sq)
        intercept_list <- rbind(intercept_list, intercept)
        comp1_list <- rbind(comp1_list, comp1) 
        comp2_list <- rbind(comp2_list, comp2)
        ratio_list <- rbind(ratio_list, ratio)
        
        incProgress(0.1, detail = paste("Part", i, "of", ncol(mixed_data)))
      }
    })			   			
    df1 <- data.frame(comp1_list, comp2_list, intercept_list, ratio_list,r_sq_list)
    colnames(df1) <- c(paste("Coeff of", name1()), paste("Coeff of", name2()), paste("Intercept"), paste("Ratio of", name1(), "to", name2()), "R-Squared")
    
    df2 <- data.frame(color1, color2, wavelengths, mixed_data)
    colnames(df2) <- c("color 1", "color 2", "Wavelengths", "Mixed_Data")		
    
    list(df1,df2) 
  })
  
  #Outputted Plots
  output$color1_plot <- renderPlot({
    df1 <- mydata()[[1]]
    plot(df1[[1]], type = "l", main = "Component 1 Coefficient Plot", xlab = "Time Points", ylab = paste("Coefficient of ", name1()), col = "blue")
  })
  output$color2_plot <- renderPlot({
    df1 <- mydata()[[1]]
    plot(df1[[2]], type = "l", main = "Component 2 Coefficient Plot", xlab = "Time Points", ylab = paste("Coefficient of ", name2()), col = "blue")
  })
  output$ratio_plot <- renderPlot({
    df1 <- mydata()[[1]]
    plot(df1[[4]], type = "l", main = "Ratio Plot",xlab = "Time Points", ylab = paste("Ratio of ", name1(), " to ", name2()), col = "blue")
  })
  
  #Downloadable file
  output$downloadData <- downloadHandler(
    filename = function(){
      paste(name3(), "csv", sep = ".")
    },
    content = function(file){
      write.table(mydata()[[1]], file, sep = ",", row.names = FALSE)
    }
  )
  
#Panel 2
  time <- reactive({
    as.numeric(input$time_point) +1
  })	
  output$Regression_Line <- renderPlot({
    df2 <- mydata()[[2]]
    color1 <- df2[[1]]
    color2 <- df2[[2]]
    wavelengths <- df2[[3]]
    df2 <- df2[-c(1:3)]
    #Models the predicted values against the observed data at an entered time point
    if(time() >= 1 & is.na(time()) == FALSE){
      
      obs <- as.double(df2[[time()]])
      fit <- lm(as.double(df2[[time()]]) ~ color1 + color2)
      pred <- fit$fitted.values
      u <- max(max(pred),max(obs))+50 #50 is an arbitrary value
      yRange <- range(pred, obs, u)
      
      matplot(x = wavelengths, y = obs, type = "l", col = "blue", ylim = yRange, xlab = "Wavelength (nm)", ylab = "Intensity", main = paste("Observed vs. Expected Plot at Time = ", (time()-1)))
      matlines(x = wavelengths, y = pred, type = "l",col = "red") 
    }
  })
  output$info <- renderPrint({	  
    df1 <- mydata()[[1]]
    format(t(df1[time(),]), nsmall = 3)	#Outputs the coefficients of both components, intercept, ratio, r^2 values below the plot  	  
  })
}

shinyApp(ui,server)
